#!/bin/sh
# Script to uninstall the OCALA proxy (must be run with root permissions.)
# Author: Dilip Antony Joseph
# NOTE: This script has been tested only with Fedora Linux. 
# If it works for you for some other version of Linux, please send a mail to support@ocala.cs.berkeley.edu.
#
# Reference: http://www.linux.com/howtos/HighQuality-Apps-HOWTO/boot.shtml
#
OCALA_INSTALL_DIR=/usr/local/OCALA

# The configuration files used by the installed version of the OCALA proxy are in 
# following directory.
OCALA_CONFIG_DIR=/usr/local/OCALA
# Note: DILIP: OCALA_CONFIG_DIR must be the same as OCALA_CONFIG_DIR, for now.  
# This is because OCALA_proxy is not very intelligent while looking for the directories
# containing the configuration files.

# Set the following variable to blank if you do not want to be prompted for file overwrite.
PROMPT_OVERWRITE=-i

echo "Uninstalling OCALA ..."


# Delete the OCALA_INSTALL_DIR.
if [ -d $OCALA_INSTALL_DIR ]
then
    rm -rf $PROMPT_OVERWRITE $OCALA_INSTALL_DIR
fi

# Delete the OCALA_CONFIG_DIR.
if [ -d $OCALA_CONFIG_DIR ]
then
    rm -rf $PROMPT_OVERWRITE $OCALA_CONFIG_DIR
fi

# Run chkconfig to stop automatic starting of OCALA at boot time.

# Check if chkconfig command can be located.
which chkconfig 2>&1> /dev/null
if [ $? -gt 0 ]
then
    echo "Required command chkconfig not found. Maybe you should do su - ?"
    echo "Aborting uninstall."
    exit
fi

#Run chkconfig
chkconfig --del OCALA
if [ ! $? ]
then
    echo "chkconfig failed. Unable to stop OCALA automatically start at boot time."
    exit
fi

# Stop the OCALA service
# Check if service command can be located.
which service 2>&1> /dev/null
if [ $? -gt 0 ]
then
    echo "Required command service not found. Maybe you should do su - ?"
    echo "Aborting uninstall."
    exit
fi

# Do the actual stopping
service OCALA stop


# Delete the OCALA script from /etc/init.d
rm /etc/init.d/OCALA


echo
echo
echo "OCALA successfully uninstalled."
echo

