/*
 * Created on Oct 9, 2005
 */
package proxyconfig;

import java.io.IOException;
import java.util.*;
import java.io.*;
import java.net.URL;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.swing.*;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import proxyconfig.ocd.*;

/**
 * This class reads the configuration file associated with
 * the GUI.  It also serves as a repository of GUI
 * configuration information - like background colors etc.
 * @author danjo
 */
public class GUIConfigReader extends DefaultHandler {

    /** Map of OCDs by their type. */
    private HashMap ocd_type_map;
    
    public GUIConfigReader (ProxyConfig proxyConfig) {
        
        ocd_type_map = new HashMap();

        // Parse the xml
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		
	        URL guiConfigFileURL = getClass().getResource("/gui.xml");
	        
			System.out.println("Parsing " + guiConfigFileURL.getFile());
			saxParser.parse (getClass().getResourceAsStream("/gui.xml"), handler);
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException sxe) {
		    sxe.printStackTrace();
		}
		  
    }
    
	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("GUIConfig")) {
			
			// get the value of all the attributes
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("taskWizard")) {
					}
					
	 			} // go through all the attributes - end
		  	} // process attributes -end
		
		} else if (qName.equals ("OCD")) {
		    
		    String tmpOCDType = null;
		    String tmpOCDClassFile = null;
		    
		    //	get the value of all the attributes
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
									
					if (aName.equals("type")) {
					    tmpOCDType = aValue;
					} else if (aName.equals ("class")) {
					    tmpOCDClassFile = aValue;
					}
					
	 			} // go through all the attributes - end
		  	} // process attributes -end
		
			if (tmpOCDType != null && tmpOCDClassFile != null ) {
			    
			    //Load the class.
			    try {
			        Class tmpOCDClass = Class.forName(tmpOCDClassFile);
			        
			        // Put this class into the hash table keyed by the 
			        // type of the class
			        ocd_type_map.put(tmpOCDType, tmpOCDClass);
			        
			    } catch (Exception e) {
			         JOptionPane.showMessageDialog (
			                 				null,
			                 				"Unable to load class " + tmpOCDClassFile + ".",
			                 				"OCD Load Error",
			                 				JOptionPane.ERROR_MESSAGE
			                 			);
			         
			    }
			} else {
			    JOptionPane.showMessageDialog(
			            					null, 
			            					"Both OCD type and class file name should be specified.",
			            					"Error in gui configuration file",
			            					JOptionPane.ERROR_MESSAGE
			            				);
			}
		}
	}

	/**
	 * Get a new of OCD of the specified type.
	 * @param type
	 * @return NULL if an OCD of the specified type was not found.
	 */
	public OCD getNewOCD (String type) {
	    
	    Class tmpOCDClass = (Class) ocd_type_map.get(type);
	    if (tmpOCDClass != null) {
	        try {
	            return (OCD) tmpOCDClass.newInstance();
	        } catch (Exception e) {
	            e.printStackTrace();
	            return null;
	        }
	    } else {
	        return null;
	    }
	}
	
	/**
	 * Return the list of ocd types known to this ProxyConfig GUI.
	 * @return
	 */
	public ArrayList getOCDTypeList() {
	    ArrayList retList = new ArrayList();
	    
	    Iterator itr = ocd_type_map.keySet().iterator();
	    while (itr.hasNext()) {
	        
	        retList.add ( (String) itr.next());
	    }
	    return retList;
	}
}
