/*
 * Created on Feb 9, 2005
 */
package proxyconfig;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.*;

/**
 * @author danjo
 */
public class InputChangeListener implements DocumentListener, ItemListener, ChangeListener, ListSelectionListener {
	
	/** Has any record in the collection been changed? */
	private boolean hasChanged;

	/** Is the listener enabled? The listener is disabled when the contents of the fields
	 *  representing the Record are updated by the program (say for showing the next record).
	 *  The listener is to capture only changes made by the user to the Record */
	private boolean isEnabled;
	
	/** Constructs a new listener and initializes enabled to true, hasChanged to false */
	public InputChangeListener() {
		
		hasChanged = false;
		isEnabled = true;
		
	}
	
	public void stateChanged(ChangeEvent ce) {
	    if (isEnabled) hasChanged = true;
	}
	
	/** Has any record in the collection been changed? 
	 *  @return true if any record in the collection was changed by the user, else false
	 */
	public boolean hasChanged() {
		
		return hasChanged;
		
	}

	/** Sets the changed/not changed state of the listener - changed or not changed 
	 *  @param st true - has Changed, false - not changed
	 */
	public void setChanged(boolean st) {
		
		hasChanged = st;
	}

	/** Enables or disables the listener 
	 *  @param st true - enables the listener , false disables the listener
	 */
	public void setEnabled(boolean st) {
	
		isEnabled = st;

	}

	//functions to be implemented as part of the DocumentListener interface
	public void insertUpdate(DocumentEvent de) {
		
		if(isEnabled) hasChanged = true;
				
	}

	public void removeUpdate(DocumentEvent de) {
		
		if(isEnabled) hasChanged = true;
		
		
	}

	public void changedUpdate(DocumentEvent de) {
		
		if(isEnabled) hasChanged = true;
	
	}

	//functions to be implemented as part of the ItemListener interface
	public void itemStateChanged(ItemEvent ie) {

		if(isEnabled) hasChanged = true;
		
	}


    public void valueChanged(ListSelectionEvent e) {
		if(isEnabled) hasChanged = true;
        
    }

}