/*
 * Created on Sep 24, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package proxyconfig;

import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import proxyconfig.ocd.OCD;
import proxyconfig.outboundconnections.OutboundRecord;

import javax.xml.parsers.SAXParserFactory; 
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * @author Dilip Antony Joseph
 */
public class ProxyConfigFileReader extends DefaultHandler {

	
	ProxyConfigData proxyConfigData;
	GUIConfigReader guiConfigReader;
	
	public static  ProxyConfigData loadFromFile(File diskFile, GUIConfigReader guiConfigReader) {
		
		try {
			ProxyConfigFileReader loader = new ProxyConfigFileReader(diskFile, guiConfigReader);
			ProxyConfigData tmpProxyConfigData = loader.getProxyConfigData();
						
			return tmpProxyConfigData;
		} catch (SAXParseException spe) {
			System.out.println(spe + "  line:" + spe.getLineNumber() + " Message: " + spe.getMessage() + " column: " + spe.getColumnNumber() + " uri: " + spe.getSystemId());
			return null;
		} catch (SAXException sxe) {
			System.out.println(sxe);
			return null;
		}
	}
	
	private ProxyConfigFileReader(File diskFile, GUIConfigReader guiConfigReader) throws SAXException{
		
	    this.guiConfigReader = guiConfigReader;
	    
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		 // Parse the xml
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new SAXException("Parser Configuration Error:" + e);
		}
		
		try {
			System.out.println("Parsing " + diskFile);
			saxParser.parse( diskFile, handler );
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new SAXException("IO Error while reading " + diskFile + "\n" + e1);
		}
		
	}
	
	ProxyConfigData getProxyConfigData() {
		return proxyConfigData;	
	}
	
	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("ProxyConfigFile")) {
		
			proxyConfigData = new ProxyConfigData();
			
			// get the value of all the attributes of the RefCollection
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("stdDNSRequestPort")) {
					    try {
					        proxyConfigData.stdDNSRequestPort = Integer.parseInt(aValue);
					    } catch (NumberFormatException nfe) {
					        System.out.println("Error while parsing std DNS Request port.\n" + nfe.getMessage());
					    }

					} else if (aName.equals("legacyDNSRequestPort")){
					    
					    try {
					    	proxyConfigData.legacyDNSRequestPort = Integer.parseInt(aValue);
				    	} catch (NumberFormatException nfe) {
				    	    System.out.println("Error while parsing legacy DNS Request port.\n" + nfe.getMessage());
				    	}

					} else if (aName.equals("fakeAddrRangeStart")){
						
					    proxyConfigData.fakeAddrRangeStart = aValue;
						
					} else if (aName.equals("fakeAddrRangeMask")) {
						
					    proxyConfigData.fakeAddrRangeMask = aValue;
					
					} else if (aName.equals (ProxyConfigData.MY_NAMES_CONFIG_FILE)) {
					    
					    proxyConfigData.myNameConfigFileName = aValue;
					    
					} else if (aName.equals (ProxyConfigData.OUTBOUND_CONNECTION_CONFIG_FILE)) {
					    
					    proxyConfigData.outboundConfigFileName = aValue;
					
					}
					
	 			} // go through all the attributes - end
		  	} // process attributes -end
		
		} else if (qName.equals("OC-D")) {
			
			OCD newOverlay = new OCD();
			for (int i = 0; i < attrs.getLength(); i++) {
				  	
				String aName = attrs.getLocalName(i);
				if ("".equals(aName)) aName = attrs.getQName(i);
					
				String aValue = (String) attrs.getValue(i);
					
				if (aName.equals("type")) {
					newOverlay.type = aValue;				
				} else if (aName.equals("id")){
					newOverlay.id = aValue;
				} else if (aName.equals("description")){
					newOverlay.description = aValue;
				} else if (aName.equals("active")){
				    if (aValue.toLowerCase().equals("yes")) {
				        newOverlay.isActive = true;
				    } else {
				        newOverlay.isActive = false;
				    }
				} else if (aName.equals("configFile")){
					newOverlay.configFile = aValue;
				} else if (aName.equals ("libFileName_windows")) {
				    newOverlay.libFileName_windows = aValue;

				} else if (aName.equals ("libFileName_linux")) {
				    newOverlay.libFileName_linux = aValue;

				} else if (aName.equals ("libFileName_apple")) {
				    newOverlay.libFileName_apple = aValue;
				    
				} else { 
				    
				    newOverlay.addProperty (aName, aValue);
				}
				
				
			} // go through all the attributes - end

			OCD tmpOCD = guiConfigReader.getNewOCD(newOverlay.type);
			if (tmpOCD != null) {
			    tmpOCD.init (newOverlay);
				proxyConfigData.overlayList.add (tmpOCD);
			}
			
		} 
	}
	
}
