/*
 * Created on May 25, 2005
 */
package proxyconfig;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import proxyconfig.InputChangeListener;
import proxyconfig.ProxyConfig;
import proxyconfig.ProxyConfigData;
import proxyconfig.ProxyConfigFileReader;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

import proxyconfig.certificate.CertificateAllocator;
import proxyconfig.ocd.*;
import proxyconfig.mynames.*;

/**
 * @author danjo
 */
public class ProxyConfigWizard extends JDialog implements ActionListener {

    public InputChangeListener changeListener = new InputChangeListener(); 

    JEditorPane titleLB;

    JButton prevBT, nextBT;
    
    ProxyConfig proxyConfig;
    JTabbedPane tabPane;
    
    public boolean finishFlag;
    
    WizardOverlayPanel overlayPanel;
    WizardFinishPanel finishPanel;
    
    public ProxyConfigWizard(ProxyConfig proxyConfig) {
        
        super(proxyConfig, "Welcome to Generic Proxy Configuration Wizard" , true);

        
        this.proxyConfig = proxyConfig;
        
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
    	int xInsets = 200;
    	int yInsets = 200;
    	
    	//Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension screenSize = new Dimension(1024,768);
    	
    	setResizable(false);
    	
    	setBounds(xInsets, yInsets, screenSize.width - xInsets*2, screenSize.height - yInsets * 2);
        
        Container contents = getContentPane();
        
        contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
        contents.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
        JPanel tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        //contents.add(tmpPanel);
        
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        titleLB = new JEditorPane("text/html",
                 	"<font size=\"4\"><b>Generic Proxy Configuration Step</b></font");
        titleLB.setEditable(false);
        titleLB.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(titleLB);
        titleLB.setMaximumSize(new Dimension(200,50));

        tmpPanel.add( (Box.createHorizontalGlue()));
            
        contents.add(Box.createRigidArea(new Dimension(20,20)));
        
        tabPane = new JTabbedPane();
        tabPane.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tabPane);
        tabPane.setEnabled (false);
        
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        contents.add(Box.createRigidArea(new Dimension(5,5)));
        contents.add(buttonPanel);

        JButton cancelBT = new JButton("CANCEL");
        cancelBT.setActionCommand("CANCEL");
        cancelBT.addActionListener (this);

        nextBT = new JButton ("NEXT>");
        nextBT.setActionCommand("NEXT_TAB");
        nextBT.addActionListener(this);
        
        prevBT = new JButton("<PREVIOUS");
        prevBT.setActionCommand("PREV_TAB");
        prevBT.addActionListener (this);
        
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(prevBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(nextBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(cancelBT);
        buttonPanel.add(Box.createHorizontalGlue());

        prevBT.setVisible (false);
            

        overlayPanel = new WizardOverlayPanel (proxyConfig);
        tabPane.addTab ("Overlays", overlayPanel);
   
        ListIterator itr = proxyConfig.proxyConfigData.overlayList.listIterator();
        while (itr.hasNext()) {
            OCD currOCD = (OCD) itr.next();
            WizardNamePanel wizNamePanel = new WizardNamePanel (currOCD, proxyConfig);
            tabPane.addTab (currOCD.id, wizNamePanel);
        }
        
        finishPanel = new WizardFinishPanel (this);
        tabPane.add ("Finish", finishPanel);
    }
    
    public void setStepMsg(String msg) {
        
        if (proxyConfig.wizardFlag) {
            msg = "Generic Proxy Configuration : " + msg;
        }
        
        titleLB.setText("<font face=\"arial\"><font size=\"5\"><b>" + msg + "</b></font>");

    }
    
    public void actionPerformed(ActionEvent ae) {
        
        String cmd = ae.getActionCommand();

        int maxTabIndex = tabPane.getTabCount() - 1;

        
        if (cmd.equals("NEXT_TAB")) {
            int currSelTab = tabPane.getSelectedIndex();
            if (currSelTab == maxTabIndex) {
                //We are done
                update();
                proxyConfig.saveConfigFile();
                if (finishPanel.startProxyCB.isSelected()) {
                    proxyConfig.startProxyFlag = true;
                } else {
                    proxyConfig.startProxyFlag = false;
                }
                
                if (proxyConfig.startProxyFlag) {
                    proxyConfig.startProxy();
                }

                System.exit(0);
                
            } else {
                
                               
                currSelTab ++;
                
                
                if (currSelTab > 1) {
                    
                    WizardNamePanel tmpNamePanel = (WizardNamePanel) tabPane.getSelectedComponent();
                    
                    // Read in contents from GUI
                    tmpNamePanel.update();
                    
                    if (!tmpNamePanel.myName.certOKFlag.equals (MyName.CERT_VALID)) {
                        
                        JOptionPane.showMessageDialog (proxyConfig,
                                "Certificate not specified for name \'" + tmpNamePanel.myName.name + "\'.\nThis is very dangerous.",
                                "Warning",
                                JOptionPane.WARNING_MESSAGE
                              );
                    }
                }
                
                if (currSelTab > 0 && currSelTab < maxTabIndex) {
                    
                    WizardNamePanel tmpNamePanel = (WizardNamePanel) tabPane.getComponentAt(currSelTab);
                    
                    if (!tmpNamePanel.overlay.isActive) {
                        currSelTab++;
                    }
                }
                
                

                tabPane.setSelectedIndex(currSelTab);
                
                if (currSelTab == maxTabIndex) {
                    nextBT.setText ("FINISH");
                } 
                if (currSelTab >0) {
                    prevBT.setVisible (true);
                }
            }
            
        } else if (cmd.equals("PREV_TAB")) {
            
            int currSelTab = tabPane.getSelectedIndex();
            
            
            if (currSelTab == 0) {
                ProxyConfig.beep();
            } else {
                currSelTab --;
                if (currSelTab == maxTabIndex) {
                    nextBT.setText("FINISH");
                } else {
                    nextBT.setText ("NEXT>");
                }
                
                if (currSelTab == 0) {
                    prevBT.setVisible (false);
                }
            
                if (currSelTab > 0 && currSelTab < maxTabIndex) {
                    
                    WizardNamePanel tmpNamePanel = (WizardNamePanel) tabPane.getComponent(currSelTab);
                    if (!tmpNamePanel.overlay.isActive) {
                        currSelTab--;
                    }
                }
                tabPane.setSelectedIndex(currSelTab);
            }
            
        } else if (cmd.equals("CANCEL")) {
            
            if (changeListener.hasChanged()) {
                
                int response = JOptionPane.showConfirmDialog(null, 
                       "Some configuration parameters have been modified.\nAre you sure you want to discard the changes?", "Warning", JOptionPane.YES_NO_OPTION);
                if (response == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            
            System.exit(0);
        }
        
    }


    public void update() {
        
        int numTabs = tabPane.getTabCount();
        for (int i = 1; i < numTabs - 1; i ++) {
            
            WizardNamePanel tmpNamePanel = (WizardNamePanel) tabPane.getComponentAt(i);
            tmpNamePanel.update();
        }
    }
    
    public boolean checkFields () {
        
        return true;
    }
     
    void showAdvancedConfig() {
        
        update();
        proxyConfig.myNamePanel.updateMyNameList();
        setVisible(false);
        
        proxyConfig.show();
    }
}
