/*
 * Created on May 18, 2005
 */
package proxyconfig.certificate;

import java.io.*;

import javax.swing.*;

import proxyconfig.mynames.MyName;

import java.awt.event.*;
import proxyconfig.*;

/**
 * @author danjo
 */
public class CertificateAllocator {

    public static final String tmpCertReqFileName = "certReq.tmp";

    public static final String tmpCertPassFileName = "certPass.tmp";

    public static String configFile = null;

    /* The constants should match those in nameAllocatorClientExitCodes.h */
    public static final int EXCODE_NORMAL_TERMINATION = 0;

    //  Codes corresponding to local error events
    public static final int EXCODE_CONFIGFILE_ERROR = 10;

    public static final int EXCODE_REQFILE_ERROR = 11;

    public static final int EXCODE_CONFIRMFILE_ERROR = 12;

    public static final int EXCODE_SEND_REQ_ERROR = 13;

    public static final int EXCODE_INVALID_CMDLINE_ARGS = 14;

    public static final int EXCODE_CONNECT_RESOLVE_FAILED = 15;

    public static final int EXCODE_CONNECT_OPENSKT_FAILED = 16;

    public static final int EXCODE_CONNECT_FAILED = 17;

    public static final int EXCODE_KEY_WRITE_FAILED = 18;

    public static final int EXCODE_KEY_LOAD_FAILED = 19;

    //  Codes corresponding to remote events
    public static final int EXCODE_NAME_AVAILABLE = 30;

    public static final int EXCODE_NAME_UNAVAILABLE = 31;

    public static final int EXCODE_NAME_CONFIRMED = 32;

    public static final int EXCODE_UNEXPECTED_REPLY = 40;

    public static final int EXCODE_SERVER_INTERNAL_ERROR = 41;

    public static final int EXCODE_POLICY_DISALLOW = 42;

    public static final int EXCODE_UNKNOWN_NAME = 43;

    public static final int EXCODE_INVALID_PASSWORD = 44;

    public static final int EXCODE_NAME_ALREADY_VERIFIED = 45;

    public static final int EXCODE_NAMEALLOCATOR_NOT_FOUND = -2000;

    public static Process nameAllocProcess;

    /**
     * Path to the name allocator executable and the working directory the
     * executable should use.
     */
    public static String nameAllocCmd = null;

    public static String getConfigFileName() {
        if (configFile == null) {
            initNameAllocCmd();
        }
        return configFile;
    }

    /**
     * Sets the nameAllocCmd and its working dir.
     */
    public static void initNameAllocCmd() {
        nameAllocCmd = null;

        String tmpNameAllocCmd = "";
        if (!ProxyConfig.isWindows()) {
            tmpNameAllocCmd = "nameAllocatorClient";
        } else {
            //windows
            tmpNameAllocCmd = "nameAllocatorClient.exe";
        }
        String tmpFileName = "./" + tmpNameAllocCmd;

        File tmpFile = new File(tmpFileName);
        if (tmpFile.exists()) {
            nameAllocCmd = tmpFileName;
            configFile = "nameAllocator.config";

        } else {

            tmpFileName = "./security/nameallocator/" + tmpNameAllocCmd;
            tmpFile = new File(tmpFileName);
            if (tmpFile.exists()) {
                nameAllocCmd = tmpFileName;
                configFile = "security/nameallocator/nameAllocator.config";
            }
        }

    }

    public static int sendCertRequest(String name, String realName, String email) {

        nameAllocProcess = null;

        try {
            //construct the request
            BufferedWriter bw = new BufferedWriter(new FileWriter(
                    tmpCertReqFileName));
            bw.write(name + "\n");
            if (realName.trim().equals("")) {
                realName = "<Real Name Not Specified>";
            }
            bw.write(realName + "\n");
            bw.write(email + "\n");

            bw.close();

            //now invoke the name allocator program.
            String nameAllocatorCmd = getNameAllocatorCmd();
            if (nameAllocatorCmd == null) {

                JOptionPane.showMessageDialog(null,
                        "Unable to find nameAllocatorClient executable.",
                        "Error", JOptionPane.ERROR_MESSAGE);
                return EXCODE_NAMEALLOCATOR_NOT_FOUND;
            }

            String cmdToExec = nameAllocatorCmd + " -C " + getConfigFileName()
                    + " -r " + tmpCertReqFileName;
            nameAllocProcess = Runtime.getRuntime().exec(cmdToExec, null);

            //System.out.println ("cmdToExec: " + cmdToExec + "\nworkingdir:" +
            // getNameAllocatorWorkingDir());

            // Set a timer to time out the connection to name allocator after 5
            // seconds
            java.util.Timer timer = new java.util.Timer();
            timer.schedule(new java.util.TimerTask() {
                public void run() {

                    CertificateAllocator.nameAllocProcess.destroy();
                }
            }, 5000);

            try {

                nameAllocProcess.waitFor();
            } catch (InterruptedException e) {
                System.out.println(e);
            }

            int exitVal = nameAllocProcess.exitValue();

            return processExitVal(exitVal, email, name);

        } catch (IOException ioe) {
            System.out.println(ioe);
            JOptionPane.showMessageDialog(null,
                    "Unable to send certificate request for " + name + ".",
                    "Certificate Request Error", JOptionPane.ERROR_MESSAGE);
            
            return EXCODE_REQFILE_ERROR;
        }

    }

    private static int processExitVal(int exitVal, String email, String name) {
        System.out.println("exit val = " + exitVal);
        switch (exitVal) {
        	case EXCODE_CONFIGFILE_ERROR:
        	    JOptionPane.showMessageDialog(
                            null,
                            "Unable to find nameAllocatorClient configuration file (nameAllocator.config).",
                            "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_NAME_AVAILABLE:
        	    JOptionPane.showMessageDialog(null,
                    "Successfully requested certificate for \'" + name
                            + "\'.\n" + "Check your " + email
                            + " email account for the confirmation password.",
                    "Success", JOptionPane.INFORMATION_MESSAGE);
            break;

        	case EXCODE_NAME_UNAVAILABLE:
        	    JOptionPane.showMessageDialog(null, "Certificate for name \'"
                    + name + "\' UNAVAILABLE.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_REQFILE_ERROR:
        	    JOptionPane.showMessageDialog(null,
                    "Unable to open certificate request file.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_CONFIRMFILE_ERROR:
        	    JOptionPane.showMessageDialog(
                            null,
                            "Unable to create/open the confirmation message file to be sent.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_CONNECT_FAILED:
        	    JOptionPane.showMessageDialog(null,
                    "Unable to connect to the name allocator server.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_CONNECT_OPENSKT_FAILED:
        	    JOptionPane.showMessageDialog(
                            null,
                            "Error while opening socket to connect to the name allocator server.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_CONNECT_RESOLVE_FAILED:
        	    JOptionPane.showMessageDialog(
                            null,
                            "Error while resolving the address of  name allocator server.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_INVALID_CMDLINE_ARGS:
        	    JOptionPane.showMessageDialog(
                            null,
                            "Invalid commad line arguments specified to the nameAllocatorClient.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_INVALID_PASSWORD:
        	    JOptionPane.showMessageDialog(null,
                    "Password supplied for the name \"" + name
                            + "\" is invalid.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_KEY_LOAD_FAILED:
        	    JOptionPane.showMessageDialog(null,
                    "Unable to load key from disk.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_KEY_WRITE_FAILED:
        	    JOptionPane.showMessageDialog(null, "Unable to write key to disk.",
                    "Error", JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_NAME_ALREADY_VERIFIED:
        	    JOptionPane.showMessageDialog(null, "The name \"" + name
                    + "\" has already been verified before.", "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_NAME_CONFIRMED:
            /*
             * JOptionPane.showMessageDialog (null, "The name \"" + name + "\"
             * has been successfully confirmed.", "Error",
             * JOptionPane.ERROR_MESSAGE );
             */
            break;

        	case EXCODE_NAMEALLOCATOR_NOT_FOUND:
        	    JOptionPane.showMessageDialog(null, "Unable to find the nameAllocatorClient executable used to contact the name allocator server",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

       
        	case EXCODE_NORMAL_TERMINATION:
            /*
            JOptionPane.showMessageDialog(null, "Normal Termination",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            */
            break;
           
        	case EXCODE_POLICY_DISALLOW:
        	    JOptionPane.showMessageDialog(null, "The name allocator server policy currently disallows this name: " + name,
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;

        	case EXCODE_UNKNOWN_NAME:
        	    JOptionPane.showMessageDialog(null, "You sent a confirmation message for an unknown name: " + name,
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;
            
        	case EXCODE_UNEXPECTED_REPLY:
        	    JOptionPane.showMessageDialog(null, "The name allocator server responded in an unexpected fashion.",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;
            
        	case EXCODE_SERVER_INTERNAL_ERROR:
        	    JOptionPane.showMessageDialog(null, "This operation faile due to an internal error at the name allocation server.",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;
           
        	case EXCODE_SEND_REQ_ERROR:
        	    JOptionPane.showMessageDialog(null, "Unable to send request to the name allocator server.",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            break;
            
        	default:
        	    JOptionPane.showMessageDialog(null, "Name allocator client return an unknown exit code.",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
        }
        
        return exitVal;
    }

    public static int sendCertConfirm(String name, String password) {

        try {

            BufferedWriter bw = new BufferedWriter(new FileWriter(
                    tmpCertPassFileName));
            bw.write(name + "\n");
            bw.write(password + "\n");

            bw.close();

            //now invoke the name allocator program.
            String nameAllocatorCmd = getNameAllocatorCmd();
            if (nameAllocatorCmd == null) {

                JOptionPane.showMessageDialog(null,
                        "Unable to find nameAllocatorClient executable.",
                        "Error", JOptionPane.ERROR_MESSAGE);
                return EXCODE_NAMEALLOCATOR_NOT_FOUND;
            }

            String cmdToExec = nameAllocatorCmd + " -C " + getConfigFileName()
                    + " -c " + tmpCertPassFileName;
            Process p = Runtime.getRuntime().exec(cmdToExec, null);
            try {

                p.waitFor();
            } catch (InterruptedException e) {
                System.out.println(e);
            }

            int exitVal = p.exitValue();

            return processExitVal(exitVal, null, name);
        } catch (IOException ioe) {
            System.out.println(ioe);
            JOptionPane
                    .showMessageDialog(null,
                            "Unable to send certificate confirmation for "
                                    + name + ".",
                            "Certificate Confirmation Error",
                            JOptionPane.ERROR_MESSAGE);
            return EXCODE_REQFILE_ERROR;
        }
    }

    /**
     * Check if the certificate of this name is valid. If the certificate is
     * valid, it returns an empty string. Else returns an appropriate error
     * message. Also sets the specified myNames certOKFlag as a sideeffect.
     * 
     * @param tmpName
     * @return
     */
    public static String checkCertificate(MyName tmpName) {

        String certMsg = "";

        if (tmpName.certFileName != null && tmpName.privKeyFileName != null) {

            File tmpCertFile = new File(tmpName.certFileName);
            if (!tmpCertFile.exists()) {
                certMsg += "Certificate file " + tmpCertFile.getPath()
                        + " does not exist.\n";
                tmpName.certOKFlag = MyName.CERT_NOT_FOUND;
            }

            File tmpPrivKeyFile = new File(tmpName.privKeyFileName);
            if (!tmpPrivKeyFile.exists()) {
                certMsg += "Private key file " + tmpPrivKeyFile.getPath()
                        + " does not exist.\n";
                tmpName.certOKFlag = MyName.CERT_NOT_FOUND;
            }

            if (!certMsg.equals("")) {
                certMsg += "\nSince the certificate for name "
                        + tmpName.name
                        + " has not been "
                        + "correctly setup, \npeople will not be able to contact you on this name securely."
                        + "\nDo you want to specify the certificate again?";

            } else {
                // Check validity of certificates
                tmpName.certOKFlag = MyName.CERT_VALID;

            }
        } else {
            certMsg += "\nSince the certificate for name "
                    + tmpName.name
                    + " has not been "
                    + "correctly setup, \npeople will not be able to contact you on this name securely."
                    + "\nDo you want to specify the certificate again?";
        }

        return certMsg;
    }

    public static String getNameAllocatorCmd() {

        if (nameAllocCmd == null) {
            initNameAllocCmd();
        }
        return nameAllocCmd;
    }

}
