/*
 * Created on May 18, 2005
 */
package proxyconfig.certificate;


import proxyconfig.mynames.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import proxyconfig.InputChangeListener;
import proxyconfig.ProxyConfig;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * @author danjo
 */
public class CertificateWizard extends JDialog implements ActionListener {

    String privKeyFileName;
    String certFileName;
    
    public InputChangeListener changeListener = new InputChangeListener(); 

    JEditorPane titleLB;
    JButton prevBT, nextBT;
    
    JTabbedPane tabPane;
    
    public boolean finishFlag;

    CertificateRequestPanel certificateRequestPanel;
    CertificateConfirmPanel certificateConfirmPanel;
    CertificateCompletePanel certificateCompletePanel;
    
    public CertificateWizard (ProxyConfig proxyConfig, String name) {
        
        super(proxyConfig, "Certificate Wizard", true);
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
    	int xInsets = 200;
    	int yInsets = 200;
    	
    	//Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension screenSize = new Dimension(1024,768);
    	
    	setResizable(false);
    	
    	setBounds(xInsets, yInsets, screenSize.width - xInsets*2, screenSize.height - yInsets * 2);
        
        Container contents = getContentPane();
        
        contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
        contents.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
        JPanel tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tmpPanel);
        
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        titleLB = new JEditorPane("text/html",
                 	"<font size=\"4\"><b>Certificate Wizard</b></font");
        titleLB.setEditable(false);
        titleLB.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(titleLB);
        titleLB.setMaximumSize(new Dimension(200,50));

        tmpPanel.add( (Box.createHorizontalGlue()));
            
        contents.add(Box.createRigidArea(new Dimension(20,20)));
        
        tabPane = new JTabbedPane();
        tabPane.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tabPane);
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        contents.add(Box.createRigidArea(new Dimension(5,5)));
        contents.add(buttonPanel);

        JButton cancelBT = new JButton("CANCEL");
        cancelBT.setActionCommand("CANCEL");
        cancelBT.addActionListener (this);

        nextBT = new JButton ("NEXT");
        nextBT.setActionCommand("NEXT_TAB");
        nextBT.addActionListener(this);
        
        prevBT = new JButton("PREVIOUS");
        prevBT.setActionCommand("PREV_TAB");
        prevBT.addActionListener (this);
        
        
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(prevBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(nextBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(cancelBT);
        buttonPanel.add(Box.createHorizontalGlue());

        prevBT.setEnabled(false);
        
        certificateRequestPanel = new CertificateRequestPanel ();
        tabPane.addTab ("Request Certificate", certificateRequestPanel);
        setStepMsg ("Request a certificate from the Certificate Authority", 1);

        certificateRequestPanel.nameTF.setText(name);
        certificateRequestPanel.setFocus();
        
        certificateConfirmPanel = new CertificateConfirmPanel ();
        tabPane.addTab ("Confirm Certificate", certificateConfirmPanel);
        setStepMsg ("Enter Confirmation Password", 2);
    
        certificateCompletePanel  = new CertificateCompletePanel();
        tabPane.addTab ("Done", certificateCompletePanel);
        setStepMsg ("Certificate Wizard Done", 2);
    
        tabPane.setEnabled(false);
        
        setStepMsg ("Request a certificate from the Certificate Authority", 1);
       

        addWindowListener( new WindowAdapter() {
                                public void windowOpened( WindowEvent e ) {
                                    certificateRequestPanel.setFocus();
                                }
                        }
         );
        show();
        changeListener.setEnabled(true);
    }
   
    public void setStepMsg(String msg, int stepNum) {
        
        if (stepNum >= 0) {
            msg = "Step " + stepNum + " : " + msg;
        }
        
        titleLB.setText("<font face=\"arial\"><font size=\"5\"><b>" + msg + "</b></font>");

    }
    
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        
        if (cmd.equals("NEXT_TAB")) {
            int currSelTab = tabPane.getSelectedIndex();
            int maxTabIndex = tabPane.getTabCount() - 1;
            //System.out.println ("currSelTab = " + currSelTab  + " max=" + maxTabIndex + "\n");
            if (currSelTab == maxTabIndex) {
                ProxyConfig.beep();
            } else {
                currSelTab ++;
                if (currSelTab == 1) {
                    if (certificateRequestPanel.checkFields()) {
                        
                        certificateConfirmPanel.nameTF.setText(certificateRequestPanel.nameTF.getText());
                        certificateConfirmPanel.realNameTF.setText(certificateRequestPanel.realNameTF.getText());
                        certificateConfirmPanel.emailTF.setText(certificateRequestPanel.emailTF.getText());
                        certificateConfirmPanel.setFocus();
                        
                        if (CertificateAllocator.sendCertRequest(
                                	certificateRequestPanel.nameTF.getText(),
                                	certificateRequestPanel.realNameTF.getText(),
                                	certificateRequestPanel.emailTF.getText()
                                ) != CertificateAllocator.EXCODE_NAME_AVAILABLE) {
                            
                            currSelTab --;

                            // We close this dialog.  We need to specify another name.
                            finishFlag = false;
                            setVisible(false);
                        }
                        
                    } else {
                        currSelTab = 0;
                    }
                } else if (currSelTab == 2) {
                    
                    if (certificateConfirmPanel.checkFields()) {
                        int retVal = CertificateAllocator.sendCertConfirm (
                                	certificateConfirmPanel.nameTF.getText(),
                                	certificateConfirmPanel.passwordTF.getText()
                                );
                        
                        if (retVal == CertificateAllocator.EXCODE_NAME_CONFIRMED) {
                            String tmpName = certificateConfirmPanel.nameTF.getText();
                            
                            File certFile = new File ("my.cert");
                            File privKeyFile = new File ("my.privkey");
                            File pubKeyFile = new File ("my.pubkey");
                            
                            File newCertFile = new File ("config_files/certificates/" + tmpName + ".cert");
                            File newPrivKeyFile = new File ("config_files/certificates/" + tmpName + ".privkey");
                            File newPubKeyFile = new File ("config_files/certificates/" + tmpName + ".pubkey");
                            
                            certFile.renameTo(newCertFile);
                            privKeyFile.renameTo(newPrivKeyFile);
                            pubKeyFile.renameTo(newPubKeyFile);
                            
                            certFileName = newCertFile.getPath();
                            privKeyFileName = newPrivKeyFile.getPath();
                            
                            certificateCompletePanel.setSuccessMessage(
                                    tmpName, certFileName, privKeyFileName);
                            
                            changeListener.setChanged(true);
                            
                        } else if (retVal == CertificateAllocator.EXCODE_INVALID_PASSWORD) {
                            certificateCompletePanel.setMessage ("<b>Password for name \'" + certificateConfirmPanel.nameTF.getText() + "\' <font color=\"red\">INCORRECT</font>.</b>");
                        } else if (retVal == CertificateAllocator.EXCODE_NAMEALLOCATOR_NOT_FOUND) {
                            certificateCompletePanel.setMessage ("<b>Unable to find nameAllocatorClient executable.</b>");
                        } else if (retVal == CertificateAllocator.EXCODE_CONFIGFILE_ERROR) {
                            certificateCompletePanel.setMessage ("<b>Unable to find nameAllocatorClient configuration file</b>");
                        } else if (retVal == CertificateAllocator.EXCODE_CONNECT_FAILED
                                || retVal == CertificateAllocator.EXCODE_CONNECT_OPENSKT_FAILED
                                || retVal == CertificateAllocator.EXCODE_CONNECT_RESOLVE_FAILED
                               ) {
                            certificateCompletePanel.setMessage ("<b>Unable to contact Certificate Authority.</b>");
                        } else {
                            certificateCompletePanel.setMessage("<b>nameAllocatorClient returned unknown exit code " + retVal + ".</b>");
                        }
                    } else {
                        currSelTab --;
                    }
                }
                tabPane.setSelectedIndex(currSelTab);
            }

            if (currSelTab == maxTabIndex) {
                nextBT.setText("FINISH");
                nextBT.setActionCommand ("FINISH");
                
            } else {
                nextBT.setText ("NEXT");
                nextBT.setActionCommand ("NEXT_TAB");
                
            }
            
            if (currSelTab != 0) {
                prevBT.setEnabled(true);
            }
            
        } else if (cmd.equals("PREV_TAB")) {
            
            int currSelTab = tabPane.getSelectedIndex();
            if (currSelTab == 0) {
                ProxyConfig.beep();
            } else {
                currSelTab --;
                tabPane.setSelectedIndex(currSelTab);
            }
            if (currSelTab == 0) {
                prevBT.setEnabled(false);
            }
            if (currSelTab < tabPane.getTabCount() - 1) {
                nextBT.setText("NEXT");
                nextBT.setActionCommand ("NEXT_TAB");
            }
            
        } else if (cmd.equals("FINISH")) {
            
            finishFlag = true;
            setVisible(false);
        } else if (cmd.equals("CANCEL")) {
            
            if (changeListener.hasChanged()) {
                
                int response = JOptionPane.showConfirmDialog(null, 
                       "Some configuration parameters have been modified.\nAre you sure you want to discard the changes?", "Warning", JOptionPane.YES_NO_OPTION);
                if (response == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            
            setVisible(false);
            finishFlag = false;
        }
        
    }

  
    public String getCertificateFileName () {
        return certFileName;
    }
     
    public String getPrivKeyFileName () {
        return privKeyFileName;
    }



}
