/*
 * Created on Jan 22, 2005
 */
package proxyconfig.i3;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


import javax.xml.parsers.SAXParserFactory; 
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * @author danjo
 */
public class BasicI3ConfigFileReader extends DefaultHandler {
	
    I3OCD overlay;

	BasicI3ConfigFileReader(I3OCD overlay) throws SAXException{
	
	    this.overlay = overlay;
	    
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		 // Parse the xml
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new SAXException("Parser Configuration Error:" + e);
		}
		
		try {
			System.out.println("Parsing " + overlay.configFile);
			saxParser.parse( overlay.configFile, handler );
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new SAXException("IO Error while reading " + overlay.configFile + "\n" + e1);
		}
		
	}

	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("I3ConfigFile")) {
		
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("version")) {
					    overlay.i3Version = aValue;

					} 
				} // go through all the attributes - end
		  	} // process attributes -end
		} else 	if (qName.equals("I3ServerDetails")) {
			
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("UseTCP")) {
					    if (aValue.toLowerCase().equals("true") || aValue.toLowerCase().equals("yes")) {
					        overlay.i3UseTCP = true;
					    } else {
					        overlay.i3UseTCP = false;
					    }
					   
					} else if (aName.equals("UsePing")){
					    if (aValue.toLowerCase().equals("true") || aValue.toLowerCase().equals("yes")) {
					        overlay.i3UsePing = true;
					    } else {
					        overlay.i3UsePing = false;
					    }
					} else if (aName.equals("ServerListURL")) {
					    overlay.statusURL = aValue;
					}					
	 			} // go through all the attributes - end
		  	} // process attributes -end
		} else 	if (qName.equals("I3Server")) {

			I3Server tmpServer = new I3Server();
			overlay.i3KnownServerList.add(tmpServer);

			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);

					
					if (aName.equals("IPAddress")) {
					    tmpServer.ipAddress = aValue;
					
					} else if (aName.equals("PortNum")){
					    
					    try {
					        tmpServer.portNum = Integer.parseInt(aValue);
					    } catch (NumberFormatException nfe) {
					        System.out.println("Invalid i3 port number : " + aValue);
					    }
					} else if (aName.equals("I3Id")){
						
					    tmpServer.i3Id = aValue;
						
					} 
				} // go through all the attributes - end
		  	} // process attributes -end
		}

	}
	

}
