/*
 * Created on Jan 21, 2005
 */
package proxyconfig.i3;

import java.util.*;

import java.io.*;
import javax.swing.*;

import proxyconfig.*;
import proxyconfig.ocd.*;
import proxyconfig.mynames.*;

/**
 * @author danjo
 */
public class I3OCD extends OCD {

    public I3WizardDialog i3Wizard;
    
    I3AddressBook addressBook;
    String addressBookFileName;
    
    String statusURL;

    boolean enableShortcut;
    boolean enableFastConnection;
    
    int localPortNum;
    
    //from i3basic config file
    boolean i3UseTCP;
    boolean i3UsePing;
    String i3Version;
    ArrayList i3KnownServerList;
    
    public static final String ADDRESS_BOOK_FILENAME = "I3_AddressBook";
    public static final String LOCAL_PORT_NUMBER = "I3_LocalPortNum";
    public static final String ENABLE_SHORTCUT = "I3_EnableShortcut";
    public static final String ENABLE_FAST_CONNECTION = "I3_EnableFastConnectionStartup";
    
    
    public static final String I3_ID = "i3Id";
    public static final String I3_TRIGGER_TYPE = "i3TriggerType";
    public static final String I3_KEY = "i3Key";
    public static final String I3_ID_STACK = "i3IdStack";
    public static final String I3_NAME_STACK = "i3NameStack";
    public static final String I3_TRIGGER_TOTAL_BW = "i3TriggerTotalBW";
    public static final String I3_TRIGGER_TOTAL_CONNECTIONS = "i3TriggerTotalConnections";
    
    public I3OCD () {
        i3KnownServerList = new ArrayList();
        type = OCD.OCD_TYPE_I3;
        isActive = true;
        addressBook = new I3AddressBook();

    }
    
    public void init (OCD o) {
        super.init (o);
      
        this.libFileName_windows = "i3OCD_windows.dll";
        this.libFileName_linux = "./libI3OCD.so";
            
        addressBookFileName = getProperty (ADDRESS_BOOK_FILENAME);
        
        try {
            localPortNum = Integer.parseInt (getProperty (LOCAL_PORT_NUMBER));
        } catch (NumberFormatException nfe) {
            System.out.println("Invalid i3 local port number " + getProperty (LOCAL_PORT_NUMBER));
        }
    
        if (getProperty (ENABLE_SHORTCUT).toLowerCase().equals ("true") 
                || getProperty (ENABLE_SHORTCUT).toLowerCase().equals ("yes")) {
            
            enableShortcut = true;
        } else {
            enableShortcut = false;
        }
        if (getProperty (ENABLE_FAST_CONNECTION).toLowerCase().equals ("true") 
                || getProperty (ENABLE_FAST_CONNECTION).toLowerCase().equals ("yes")) {
            
            enableFastConnection = true;
        } else {
            enableFastConnection = false;
        }
        
        i3KnownServerList = new ArrayList();
        
        loadDataFromFile();
    }
    
    public I3OCD(I3OCD o) {

        id = o.id;
        type = o.type;
        description = o.description;
        configFile = o.configFile;
        libFileName_windows = o.libFileName_windows;
        libFileName_linux = o.libFileName_linux;
        
        isActive = o.isActive;
        
        i3Wizard = o.i3Wizard;
        
        statusURL = o.statusURL;
        
        addressBookFileName = o.addressBookFileName;
        
        localPortNum = o.localPortNum;
        
        i3UseTCP = o.i3UseTCP;
        i3UsePing = o.i3UsePing;
        i3Version = o.i3Version;

                
        i3KnownServerList = new ArrayList();
        ListIterator itr = o.i3KnownServerList.listIterator();
        while (itr.hasNext()) {
            i3KnownServerList.add((I3Server) itr.next());
        }
        
        addressBook = new I3AddressBook(o.addressBook);

    }
    
    void loadDataFromFile() {
        try {
            File tmpFile = new File(configFile);
            if ( ! tmpFile.exists()) {
                JOptionPane.showMessageDialog(null, 
                        "I3 Config File " + configFile + " not found",
                        "File Not Found",
                        JOptionPane.ERROR_MESSAGE
                       );
            } else {
            
                new BasicI3ConfigFileReader(this);
                
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, 
                    "Error while parsing the i3 proxy configuration file " + configFile + "\n" + e);
            
        }
        
        // load address book
        addressBook = new I3AddressBook();
        addressBook.loadFromFile(addressBookFileName);
 
    }
   
    
    public void writeToDisk() throws IOException {
        writeBasicI3ConfigFile();
        writeAddressBookFile();
    }
    
    private void writeAddressBookFile() throws IOException {
        
        BufferedWriter bw = new BufferedWriter(new FileWriter(addressBookFileName));
        
        ListIterator itr = addressBook.addressList.listIterator();
        while (itr.hasNext()) {
            
            I3AddressBookEntry currEntry = (I3AddressBookEntry) itr.next();
            bw.write(currEntry.name + "\t" + currEntry.i3Id + "\n");
        }
        bw.close();
    }
    
    private void writeBasicI3ConfigFile() throws IOException {
//      write the basic i3 config file.
        BufferedWriter bw = new BufferedWriter (new FileWriter(configFile));
        bw.write("<?xml version=\"1.0\"?>\n");
        bw.write("<I3ConfigFile version=\"" + i3Version + "\">\n");
        
        bw.write("\t<I3ServerDetails\n");
        bw.write("\t\tUsePing=\"" + i3UsePing + "\"\n");
        bw.write("\t\tUseTCP=\"" + i3UseTCP + "\"\n");
        bw.write("\t\tServerListURL=\"" + ProxyConfigData.entitizeXML(statusURL) + "\"\n");
        bw.write("\t>\n");
        
        ListIterator itr = i3KnownServerList.listIterator();
        while (itr.hasNext()) {
            I3Server tmpServer = (I3Server) itr.next();
            bw.write("\t\t<I3Server\n");
            bw.write("\t\tIPAddress=\"" + ProxyConfigData.entitizeXML(tmpServer.ipAddress) + "\"\n");
            bw.write("\t\tPortNum=\"" + tmpServer.portNum + "\"\n");
            bw.write("\t\tI3Id=\"" + ProxyConfigData.entitizeXML(tmpServer.i3Id) + "\"\n");
            bw.write("\t\t/>\n");
        }
        bw.write("\t</I3ServerDetails>\n");
        
        bw.write("</I3ConfigFile>\n");
        
        bw.close();
    }


    public OCDWizardDialog getNewWizard (ProxyConfig proxyConfig) {
        
        i3Wizard =  new I3WizardDialog (proxyConfig, this);
        return (OCDWizardDialog) i3Wizard;
        
    }

    public void updateFromWizard() {
        if (i3Wizard != null) {
            i3Wizard.update();
        }
    }
    
    public void initNameInNameWizard (MyName tmpName) {
        tmpName.ocdSpecificProperties.put(I3OCD.I3_TRIGGER_TYPE, "public");
        
    }
    
    public OCDNameEditPanel getNewNameEditPanel (MyName tmpName) {
	    return (OCDNameEditPanel) new I3NameEditPanel (tmpName);
	}
}
