/*
 * Created on Jan 24, 2005
 */
package proxyconfig.i3;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import proxyconfig.InputChangeListener;
import proxyconfig.ProxyConfig;
import proxyconfig.ProxyConfigData;
import proxyconfig.ProxyConfigFileReader;
import proxyconfig.ocd.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

/**
 * @author danjo
 */
public class I3WizardDialog extends OCDWizardDialog implements ActionListener, ChangeListener {

    public InputChangeListener changeListener = new InputChangeListener(); 

    JEditorPane titleLB;

    JButton prevBT, nextBT;
    
    ProxyConfig proxyConfig;
    I3OCD overlay;
    
    I3OCDPanel i3OverlayPanel;

    JTabbedPane tabPane;
    
    public boolean finishFlag;
    
    I3ServerPanel i3ServerPanel;
    NameResolutionPanel nameResolutionPanel;
    I3WizardFinishPanel finishPanel;

    private String logoFileName = "i3_logo_big.gif";
       
    public I3WizardDialog(ProxyConfig proxyConfig, I3OCD overlay) {
        
        super(proxyConfig, "I3 Configuration Wizard");

        
        this.proxyConfig = proxyConfig;
        this.overlay = overlay;
        
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
    	int xInsets = 200;
    	int yInsets = 200;
    	
    	//Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension screenSize = new Dimension(1024,768);
    	
    	setResizable(false);
    	
    	setBounds(xInsets, yInsets, screenSize.width - xInsets*2, screenSize.height - yInsets * 2);
        
        Container contents = getContentPane();
        
        contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
        contents.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
        JPanel tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tmpPanel);
        
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        URL logoURL = getClass().getResource("/" + logoFileName);
        tmpPanel.add(new JLabel(new ImageIcon(Toolkit.getDefaultToolkit().getImage(logoURL))));

        titleLB = new JEditorPane("text/html",
                 	"<font size=\"4\"><b>I3 Proxy Configuration Step</b></font");
        titleLB.setEditable(false);
        titleLB.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(titleLB);
        titleLB.setMaximumSize(new Dimension(200,50));

        tmpPanel.add( (Box.createHorizontalGlue()));
            
        contents.add(Box.createRigidArea(new Dimension(20,20)));
        
        tabPane = new JTabbedPane();
        tabPane.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tabPane);
        
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        contents.add(Box.createRigidArea(new Dimension(5,5)));
        contents.add(buttonPanel);

        JButton cancelBT = new JButton("CANCEL");
        cancelBT.setActionCommand("CANCEL");
        cancelBT.addActionListener (this);

        if ( proxyConfig.wizardFlag) {
            nextBT = new JButton ("NEXT");
            nextBT.setActionCommand("NEXT_TAB");
            nextBT.addActionListener(this);
        
            prevBT = new JButton("PREVIOUS");
            prevBT.setActionCommand("PREV_TAB");
            prevBT.addActionListener (this);
        
        
            buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
            buttonPanel.add(Box.createHorizontalGlue());
            buttonPanel.add(prevBT);
            buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
            buttonPanel.add(nextBT);
            buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
            buttonPanel.add(cancelBT);
            buttonPanel.add(Box.createHorizontalGlue());

            prevBT.setEnabled(false);
            
        } else {
            JButton okBT = new JButton("OK");
            okBT.setActionCommand("FINISH");
            okBT.addActionListener(this);

            buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
            buttonPanel.add(Box.createHorizontalGlue());
            buttonPanel.add(okBT);
            buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
            buttonPanel.add(cancelBT);
            buttonPanel.add(Box.createHorizontalGlue());

        }

        

        i3OverlayPanel = new I3OCDPanel(overlay, proxyConfig, changeListener);
        tabPane.addTab("General", i3OverlayPanel);
        setStepMsg("General configuration information", 1);
        
        tabPane.addChangeListener(this);
        
        i3ServerPanel = new I3ServerPanel(overlay, changeListener);
        tabPane.addTab ("I3 Servers", i3ServerPanel);
        setStepMsg("I3 Server Information", 2);
        
        nameResolutionPanel = new NameResolutionPanel(overlay, changeListener);
        tabPane.add("Address Book", nameResolutionPanel);
        setStepMsg ("I3 Name Resolution", 3);
        
        if (proxyConfig.wizardFlag) {
            finishPanel = new I3WizardFinishPanel(overlay);
        	tabPane.add("Done", finishPanel);
        }
        
        changeListener.setEnabled(true);
    }
    
    public void setStepMsg(String msg, int stepNum) {
        
        if (proxyConfig.wizardFlag && stepNum >= 0) {
            msg = "I3 Proxy Configuration - Step " + stepNum + " : " + msg;
        }
        
        titleLB.setText("<font face=\"arial\"><font size=\"5\"><b>" + msg + "</b></font>");

    }
    
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        
        if (cmd.equals("NEXT_TAB")) {
            int currSelTab = tabPane.getSelectedIndex();
            int maxTabIndex = tabPane.getTabCount() - 1;
            if (currSelTab == maxTabIndex) {
                ProxyConfig.beep();
            } else {
                currSelTab ++;
                tabPane.setSelectedIndex(currSelTab);
            }
            
        } else if (cmd.equals("PREV_TAB")) {
            
            int currSelTab = tabPane.getSelectedIndex();
            if (currSelTab == 0) {
                ProxyConfig.beep();
            } else {
                currSelTab --;
                tabPane.setSelectedIndex(currSelTab);
            }
            
        } else if (cmd.equals("FINISH")) {
            
            finishFlag = true;
            setVisible(false);
        } else if (cmd.equals("CANCEL")) {
            
            if (changeListener.hasChanged()) {
                
                int response = JOptionPane.showConfirmDialog(null, 
                       "Some configuration parameters have been modified.\nAre you sure you want to discard the changes?", "Warning", JOptionPane.YES_NO_OPTION);
                if (response == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            
            setVisible(false);
            finishFlag = false;
        }
        
    }

    public void stateChanged(ChangeEvent e) {
        switch (tabPane.getSelectedIndex()) {
        	case 0:
        	    setStepMsg("General Configuration", 1);
        	    break;
    	
        	case 1:
        	    setStepMsg("I3 Servers", 2);
        	    break;
    	
        	case 2:
        	    setStepMsg("Name Resolution", 3);
        	    break;
    	
        	case 4:
        	    setStepMsg("I3 Configuration DONE!", -1);
        	break;
        	
        	default:
        	    setStepMsg("I3 Configuration STEP", -1);
        }
    
        int currSelTab = tabPane.getSelectedIndex();
        
        if (proxyConfig.wizardFlag) {
            if (currSelTab == tabPane.getTabCount() - 1) {
            	nextBT.setText("FINISH");
            	nextBT.setActionCommand("FINISH");
        	} else {
            	nextBT.setText("NEXT");
            	nextBT.setActionCommand("NEXT_TAB");
        	}
    
        	if (currSelTab == 0) {
            	prevBT.setEnabled(false);
        
        	} else {
            	prevBT.setEnabled(true);
        	}
        }

    }
    
    public void update() {
        i3ServerPanel.update();
        i3OverlayPanel.update();
        nameResolutionPanel.update();
    }
    
    public boolean checkFields () {
        
        if (!finishFlag) {
            //cancel was hit
            return true;
        }
        
        if (!i3OverlayPanel.checkFields()) {
            return false;
        }
        if (!i3ServerPanel.checkFields()) {
            return false;
        }
        return true;
    }
     
    
    public boolean hasChanged() {
        
        return changeListener.hasChanged();
    }
    
    public boolean isOK() {
        
        return finishFlag;
    }
}
