/*
 * Created on May 16, 2005
 */
package proxyconfig.mynames;

import proxyconfig.*;

import java.io.*;
import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory; 
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * @author Dilip Antony Joseph
 */
public class MyNameConfigFileReader extends DefaultHandler {

    ArrayList myNameList;
    
    public MyNameConfigFileReader(File diskFile, ArrayList myNameList) throws SAXException{
		
        this.myNameList = myNameList;
        
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		 // Parse the xml
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new SAXException("Parser Configuration Error:" + e);
		}
		
		try {
			System.out.println("Parsing " + diskFile);
			saxParser.parse( diskFile, handler );
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new SAXException("IO Error while reading " + diskFile + "\n" + e1);
		}
		
	}
	
	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("MyName")) {
		    
		    MyName currName = new MyName();
		    
			// get the value of all the attributes of the RefCollection
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("name")) {
					    currName.name = aValue;
					} else if (aName.equals("ocdId")){
					    currName.ocdId = aValue;
					} else if (aName.equals("active")){
					    if (aValue.toLowerCase().trim().equals ("yes") ||
					            aValue.toLowerCase().trim().equals("true")) {
					        currName.active = true;
					    } else {
					        currName.active = false;
					    }
					} else if (aName.equals("certificate")) {
					    StringTokenizer st = new StringTokenizer (aValue, ":");
					    if (st.countTokens() == 2) {
					        currName.certFileName = st.nextToken();
					        currName.privKeyFileName = st.nextToken();
					    } else {
					        System.out.println ("Malformed certificate string \"" + aValue + "\".");
					    }
					} else {
					    currName.ocdSpecificProperties.put (aName, aValue);
					}
					
	 			} // go through all the attributes - end
		  	}
			
			if (currName.active) {
			    myNameList.add(currName);			
			}
			
		}
	}
	
}
