/*
 * Created on May 17, 2005
 */
package proxyconfig.ocd;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.util.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.event.*;

import proxyconfig.*;
import proxyconfig.certificate.CertificateWizard;
import proxyconfig.mynames.MyName;
import proxyconfig.mynames.MyNameEditPanel;
import proxyconfig.mynames.MyNameTable;
import proxyconfig.mynames.MyNameTableModel;

/**
 * @author danjo
 */
public class OCDPanel extends JPanel implements ActionListener,
        ListSelectionListener {

    ProxyConfig proxyConfig;

    JButton activateBT;

    private OCDTableModel tableModel;

    private OCDTable ocdTable;

    public OCDPanel(ProxyConfig proxyConfig) {

        this.proxyConfig = proxyConfig;

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        add(Box.createVerticalGlue());

        JEditorPane welcomeLB = new JEditorPane();
        welcomeLB.setContentType("text/html");
        welcomeLB.setEditable(false);
        welcomeLB.setMaximumSize(new Dimension(700, 200));
        welcomeLB.setBackground(ProxyConfig.BACKGROUND_COLOR);

        String welcomeStr1 = "<br>"
                + "<font face=\"arial\">"
                + "The following overlay networks are currently available in the proxy.";

        welcomeLB.setText(welcomeStr1);

        add(welcomeLB);

        add(Box.createRigidArea(new Dimension(20, 20)));

        String columns[] = { "Id", "Type", "Description" };

        tableModel = new OCDTableModel(columns);

        ocdTable = new OCDTable(tableModel);
        ocdTable.setPreferredSize(new Dimension(500, 450));
        ocdTable.getSelectionModel().addListSelectionListener(this);

        JScrollPane urlTableScroller = new JScrollPane(ocdTable);
        urlTableScroller
                .setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        add(urlTableScroller);

        ListIterator itr = proxyConfig.proxyConfigData.overlayList
                .listIterator();
        while (itr.hasNext()) {
            OCD currOverlay = (OCD) itr.next();
            tableModel.addRecord(currOverlay);

        }

        tableModel.fireTableDataChanged();

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());

        JButton addBT = new JButton("Add Overlay");
        addBT.setActionCommand("ADD_OVERLAY");
        addBT.addActionListener(this);
        buttonPanel.add(addBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));

        JButton configBT = new JButton("Configure Overlay");
        configBT.setActionCommand("CONFIG_OVERLAY");
        configBT.addActionListener(this);
        buttonPanel.add(configBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));

        activateBT = new JButton("Activate Overlay");
        activateBT.setActionCommand("ACTIVATE_OVERLAY");
        activateBT.addActionListener(this);
        buttonPanel.add(activateBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));

        JButton delBT = new JButton("Delete Overlay");
        delBT.setActionCommand("DEL_OVERLAY");
        delBT.addActionListener(this);
        buttonPanel.add(delBT);

        buttonPanel.add(Box.createHorizontalGlue());

        add(buttonPanel);
        add(Box.createVerticalGlue());

    }

    public void actionPerformed(ActionEvent ae) {

        String cmd = ae.getActionCommand();

        if (cmd.equals("ADD_OVERLAY")) {
            addOverlay();
        } else if (cmd.equals("DEL_OVERLAY")) {
            OCD currOCD = (OCD) tableModel.getRecord(ocdTable.getSelectedRow());
            if (currOCD != null) {

                int retVal = JOptionPane.showConfirmDialog(null,
                        "Are you sure you want to delete the OCD" + currOCD
                                + " ?", "Confirm Delete",
                        JOptionPane.YES_NO_OPTION);
                if (retVal == JOptionPane.YES_OPTION) {

                    //TODO Go through all outbound hops to make sure that the
                    // delete ocd is not referenced.

                    tableModel.removeRecord(currOCD);

                    proxyConfig.proxyConfigData.overlayList.remove(currOCD);
                    tableModel.fireTableDataChanged();

                    proxyConfig.changeListener.setChanged(true);
                }

            } else {
                ProxyConfig.beep();
            }

        } else if (cmd.equals("CONFIG_OVERLAY")) {
            OCD currOCD = (OCD) tableModel.getRecord(ocdTable.getSelectedRow());
            if (currOCD != null) {
                configOCD(currOCD);
            } else {
                ProxyConfig.beep();
            }
        } else if (cmd.equals("ACTIVATE_OVERLAY")) {

            OCD currOCD = (OCD) tableModel.getRecord(ocdTable.getSelectedRow());

            if (currOCD != null) {

                currOCD.isActive = true;
                activateBT.setActionCommand("INACTIVATE_OVERLAY");
                activateBT.setText("Inactivate Overlay");

                tableModel.fireTableDataChanged();
            } else {
                ProxyConfig.beep();
            }
        } else if (cmd.equals("INACTIVATE_OVERLAY")) {
            OCD currOCD = (OCD) tableModel.getRecord(ocdTable.getSelectedRow());

            if (currOCD != null) {

                currOCD.isActive = false;
                activateBT.setActionCommand("ACTIVATE_OVERLAY");
                activateBT.setText("Activate Overlay");

                tableModel.fireTableDataChanged();
            } else {
                ProxyConfig.beep();
            }

        }
    }

    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting())
            return;

        int selRow = ocdTable.getSelectionModel().getMinSelectionIndex();
        OCD currOCD = (OCD) tableModel.getRecord(selRow);
        if (currOCD != null) {
            if (currOCD.isActive) {
                activateBT.setActionCommand("INACTIVATE_OVERLAY");
                activateBT.setText("Inactivate Overlay");
            } else {
                activateBT.setActionCommand("ACTIVATE_OVERLAY");
                activateBT.setText("Activate Overlay");
            }
        }
    }

    private void configOCD(OCD currOCD) {

        OCDWizardDialog ocdWiz = currOCD.getNewWizard(proxyConfig);

        if (ocdWiz != null) {
            ocdWiz.show();
            // Did the user confirm the changes made using the wizard.
            // i.e. Cancel was not pressed.
            if (ocdWiz.isOK()) {
                // Has the user changed any parameters using the wizard?
                if (ocdWiz.hasChanged()) {
                    tableModel.fireTableDataChanged();
                    proxyConfig.changeListener.setChanged(true);
                }
            }
        } else {
            JOptionPane.showMessageDialog(null,
                    "No Wizard associated with OCD of type" + currOCD.type
                            + ".", "No Wizard Found.",
                    JOptionPane.ERROR_MESSAGE);
        }
    }

    private void addOverlay() {
        
        ArrayList tmpTypeList = proxyConfig.guiConfigReader.getOCDTypeList();
        
        Object[] possibilities = new Object[tmpTypeList.size()];
        Iterator tmpItr = tmpTypeList.iterator();
        
        int index = 0;
        while (tmpItr.hasNext()) {
            
            possibilities[index++] = (String) tmpItr.next();
        }
        
        String s = (String) JOptionPane.showInputDialog(proxyConfig,
                "What type of overlay do you want to add?",
                "Choose Overlay Type", JOptionPane.PLAIN_MESSAGE, null,
                possibilities, possibilities[0]);

        if (s != null) {

            addOCD(s);
        }
    }

    void addOCD(String ocdType) {

        OCD currOCD = proxyConfig.guiConfigReader.getNewOCD(ocdType);
        if (currOCD != null) {
            OCDWizardDialog ocdWiz = currOCD.getNewWizard (proxyConfig);
            while (true) {

                ocdWiz.show();
                if (ocdWiz.checkFields()) {
                    break;
                }
            }
            
            if (ocdWiz.isOK()) {

                if (ocdWiz.hasChanged()) {
                    ocdWiz.update();
                    proxyConfig.proxyConfigData.addOCD(currOCD);
                    tableModel.addRecord(currOCD);
                    tableModel.fireTableDataChanged();
                    proxyConfig.changeListener.setChanged(true);
                }
            }
        
        } else {
        
            JOptionPane.showMessageDialog(null,
                "No Wizard associated with OCD of type" + currOCD.type + ".", 
                "No Wizard Found.",
                JOptionPane.ERROR_MESSAGE);
        }
    }
}