/*
 * Created on May 17, 2005
 */
package proxyconfig.outboundconnections;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import javax.swing.*;

import java.util.*;

import proxyconfig.*;
import proxyconfig.ocd.OCD;
import proxyconfig.ron.*;

/**
 * @author danjo
 */
public class HopInfoEditPanel extends JPanel implements ActionListener {

    JComboBox ocdCombo;
    
    JTextField hopNameTF;
    ProxyConfig proxyConfig;
    
    JRadioButton middleBoxBT;
    JRadioButton lastHopBT;
    
    JLabel hopNameLB;
    JLabel overlayLB;
    
    HopInfoEditPanel(HopInfo currHop, ProxyConfig proxyConfig) {
        
        this.proxyConfig = proxyConfig;
        
//      The preferred Dimension of all the input fields - mainly used to enforce a common height
		Dimension minDim = new Dimension(50, 25);
		
        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
    
        setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));

        JEditorPane msgLB = new JEditorPane();
        msgLB.setContentType ("text/html");
        msgLB.setMaximumSize (new Dimension (700,60));
        msgLB.setEditable (false);
        msgLB.setText ("If this is not the last hop, you need to specify only the name of the middlebox."
                	+ "The overlay is automatically inferred from the suffix of the middlebox name."
        			);
        msgLB.setBackground(ProxyConfig.BACKGROUND_COLOR);
        add(msgLB);
        
        middleBoxBT = new JRadioButton ("MiddleBox hop");
        middleBoxBT.setActionCommand ("MIDDLE_BOX");
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(middleBoxBT);
        middleBoxBT.addActionListener(this);
        add (middleBoxBT);
        
        JPanel tmpPanel;
		
        tmpPanel = new JPanel ();
        tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
        add(tmpPanel);
        
		hopNameLB = new JLabel("Middlebox Name");
		hopNameLB.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		tmpPanel.add(hopNameLB);
		tmpPanel.add (Box.createRigidArea (new Dimension(5,5)));
		
		
	    hopNameTF = new JTextField (20);
		hopNameTF.setMinimumSize (minDim);
		tmpPanel.add(hopNameTF);

		add (Box.createRigidArea (new Dimension(20,20)));
		
		
		msgLB = new JEditorPane();
        msgLB.setContentType ("text/html");
        msgLB.setMaximumSize (new Dimension (700,60));
        msgLB.setEditable (false);
        msgLB.setText ("If this is last hop, you need to only specify the last overlay on the path.");
        msgLB.setBackground(ProxyConfig.BACKGROUND_COLOR);
        add(msgLB);
        
		lastHopBT = new JRadioButton ("Last Hop");
		lastHopBT.setActionCommand ("LAST_HOP");
		buttonGroup.add (lastHopBT);
		lastHopBT.addActionListener (this);
		add(lastHopBT);
		
		tmpPanel = new JPanel ();
        tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
        add (tmpPanel);
    
    	overlayLB = new JLabel("Overlay");
		overlayLB.setBackground(Color.WHITE);
		tmpPanel.add(overlayLB);

		ocdCombo = new JComboBox();
		ocdCombo.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		ocdCombo.setMinimumSize(minDim);
		
		tmpPanel.add(Box.createRigidArea (new Dimension(5,5)));
		tmpPanel.add(ocdCombo);

		Iterator itr = proxyConfig.proxyConfigData.overlayList.listIterator();
		while (itr.hasNext()) {
		    OCD currOCD = (OCD) itr.next();
		    ocdCombo.addItem(currOCD);
		}
			
		if (currHop != null) {
		
		    if (currHop.endPointName != null && ! currHop.endPointName.trim().equals ("")) {
		        hopNameTF.setText (currHop.endPointName);
		        middleBoxBT.setSelected(true);	
		        ocdCombo.setEnabled (false);
		        overlayLB.setEnabled (false);
		        hopNameTF.setEnabled (true);
		        hopNameLB.setEnabled (true);
		        
		    } else {
		    
		        ocdCombo.setSelectedItem(currHop.ocd);
		        lastHopBT.setSelected(true);
		        hopNameTF.setEnabled (false);
		        hopNameLB.setEnabled (false);
		        
		        ocdCombo.setEnabled (true);
		        overlayLB.setEnabled (true);
		    }
		} else {
		    middleBoxBT.setSelected(true);
		    ocdCombo.setEnabled (false);
		    overlayLB.setEnabled (false);
		    
		    hopNameLB.setEnabled (true);
		    hopNameTF.setEnabled (true);
		}
    }
    
    public Dimension getPreferredSize() {
        
        return new Dimension(600,180);
    }
    
    boolean checkFields() {
        if ( ! lastHopBT.isSelected()) {
        
           String currHopName = hopNameTF.getText();
        
        	ListIterator itr = proxyConfig.proxyConfigData.overlayList.listIterator();
        	while (itr.hasNext()) {
        	    OCD currOCD = (OCD) itr.next();

            	if (currHopName.endsWith(currOCD.id)) {
                	return true;
            	}
        	}
        }
        return false;
    }
    
    public boolean updateRecord (HopInfo rec) {
        
        if (lastHopBT.isSelected()) {
            rec.ocd = (OCD) ocdCombo.getSelectedItem();
        	rec.ocdId = rec.ocd.id;
        	return true;
        } else {
        
            String currHopName = hopNameTF.getText();
        
        	ListIterator itr = proxyConfig.proxyConfigData.overlayList.listIterator();
        	while (itr.hasNext()) {
        	    OCD currOCD = (OCD) itr.next();

            	if (currHopName.endsWith(currOCD.id)) {
                	rec.endPointName = hopNameTF.getText();
                	rec.ocd = currOCD;
                	rec.ocdId = currOCD.id;
                	return true;
            	}
        	}
        }
        return false;
    }
    

    public void actionPerformed (ActionEvent ae) {

        String cmd  = ae.getActionCommand ();
        if (cmd.equals ("MIDDLE_BOX")) {
            
            hopNameTF.setEnabled(true);
            hopNameLB.setEnabled (true);
            
            ocdCombo.setEnabled (false);
            overlayLB.setEnabled (false);
            
        } else if (cmd.equals("LAST_HOP")) {
            hopNameTF.setEnabled (false);
            hopNameLB.setEnabled (false);
            ocdCombo.setEnabled (true);
            overlayLB.setEnabled (true);        }
    }

}