/*
 * Created on May 17, 2005
 */
package proxyconfig.outboundconnections;

import proxyconfig.*;

import java.io.*;
import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory; 
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * @author Dilip Antony Joseph
 */
public class OutboundConfigFileReader extends DefaultHandler {

    ArrayList outboundRecordList;
    private OutboundRecord currOutboundRecord;
    
    public OutboundConfigFileReader(File diskFile, ArrayList outboundRecordList) throws SAXException{
		
        this.outboundRecordList = outboundRecordList;
        
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		 // Parse the xml
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new SAXException("Parser Configuration Error:" + e);
		}
		
		try {
			System.out.println("Parsing " + diskFile);
			saxParser.parse( diskFile, handler );
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new SAXException("IO Error while reading " + diskFile + "\n" + e1);
		}
		
	}
	
	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("PathInfo")) {
		    
		    currOutboundRecord = new OutboundRecord ();
		    outboundRecordList.add (currOutboundRecord);

		    if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals ("ocdDefaultFlag")) {
					    if (aValue.equals("true")) {
					        currOutboundRecord.ocdDefaultFlag = true;
					        outboundRecordList.remove(currOutboundRecord);
					    } else {
					        currOutboundRecord.ocdDefaultFlag = false;
					    }
					}
				}
		    }

		} else if (qName.equals ("Match")) {
		    
		    if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("urlPattern")) {
					    currOutboundRecord.pattern = aValue;
					} else if (aName.equals("dstPort")){
					    try {
					        currOutboundRecord.dstPort = Integer.parseInt(aValue);
					    } catch (NumberFormatException nfe) {
					        System.out.println ("Invalid destination port " + aValue + "\n");
					    }
					}					
	 			} // go through all the attributes - end
		  	}
		} else if (qName.equals ("Security")) {
		    if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("protocol")) {
					    currOutboundRecord.securityProtocol = aValue;
					} 
	 			} // go through all the attributes - end
		  	}
		
		
		} else if (qName.equals ("Compression")) {
		    if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("algo")) {
					    currOutboundRecord.compressionAlgo = aValue;
					} else if (aName.equals ("level")) {
					    try {
					        currOutboundRecord.compressionLevel = Integer.parseInt (aValue);
					    } catch (NumberFormatException nfe) {
					        System.out.println ("Invlalid compression level " + aValue + "\n");
					    }
					}
	 			} // go through all the attributes - end
		  	}
		    
		    
		} else if (qName.equals ("Hop")) {
		    HopInfo currHop = new HopInfo ();
		    currOutboundRecord.addHop (currHop);
		    
		    if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("overlayId")) {
					    currHop.ocdId = aValue;
					} else if (aName.equals ("hopEndPointName")) {
					    currHop.endPointName = aValue;
					} else if (aName.equals("authenticate")) {
					    if (aValue.toLowerCase().equals("no")) {
					        currHop.authenticateFlag = false;
				            } else {
						currHop.authenticateFlag = true;
					    }
					}
	 			} // go through all the attributes - end
		  	}
		}
	}
	
}
