/*
 * Created on Apr 12, 2006
 */
package proxyconfig.rsptunnelocd;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.*;

/**
 * @author danjo
 */
public class ConfigFileReader extends DefaultHandler {
	
    RSPTunnelOCD overlay;

	ConfigFileReader(RSPTunnelOCD overlay) throws SAXException{
	
	    this.overlay = overlay;
	    
		DefaultHandler handler = this; 
		
		// Use the default validating parser
	  	SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		
		 // Parse the xml
		SAXParser saxParser;
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new SAXException("Parser Configuration Error:" + e);
		}
		
		try {
			System.out.println("Parsing " + overlay.configFile);
			saxParser.parse( overlay.configFile, handler );
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new SAXException("IO Error while reading " + overlay.configFile + "\n" + e1);
		}
		
	}

	/** This function is called whenever start tags are detected in the input XML file
	 */
	public void startElement(String namespaceURI,
			String sName, // simple name
			String qName, // qualified name
			Attributes attrs)
	throws SAXException {
		
		if (qName.equals("RSPTunnelOCDConfigFile")) {
		
			if (attrs != null) {
				for (int i = 0; i < attrs.getLength(); i++) {
				  	
				  	String aName = attrs.getLocalName(i);
					if ("".equals(aName)) aName = attrs.getQName(i);
					
					String aValue = (String) attrs.getValue(i);
					
					if (aName.equals("myPortNum")) {
					    try {
					        overlay.myPortNum = Integer.parseInt(aValue);
					    } catch (NumberFormatException nfe) {
					        overlay.myPortNum = 0;
					        System.out.println ("Unable to parse myPortNum from " + overlay.configFile + ": " + nfe);
					        nfe.printStackTrace();
					    }
					} else if (aName.equals("rspDomainList")) {
					    StringTokenizer st = new StringTokenizer (aValue, ",");
					    while (st.hasMoreTokens()) {
					        overlay.rspDomainList.add (st.nextToken().trim());    
					    }
					    
					} else if (aName.equals("gatewayDownPolicy")) {
					    overlay.gatewayDownPolicy = aValue;
					}
				} // go through all the attributes - end
		  	} // process attributes -end
		} 
	}
	


}
